package org.q.app.config;

import java.io.IOException;
import java.util.List;
import java.util.Map;

/**
 * Interface for loading resources from a location on the file system. Resources typically have a base path and manage
 * all resources nested in the base path. Since most of the methods of this interface perform IO, the methods can throw
 * IOException's when a IO problem occurs.
 * 
 * @author Erik Lievaart
 */
public interface Resources {

	/**
	 * List all resources contained in the specified relative path to a directory.
	 * 
	 * @param path
	 *            location relative to the root of this Resources implementation.
	 * @return a List of all relative paths to the nested resources.
	 */
	public List<String> listAll(final String path) throws IOException;

	/**
	 * Read a resource and return the contents as a String.
	 * 
	 * @param path
	 *            relative path to the resource.
	 * @return the contents of the resource.
	 */
	public String loadString(String path) throws IOException;

	/**
	 * Load an ini file. The keys and values in the ini file will be used as keys and values in the returned map.
	 * 
	 * @param path
	 *            relative path to the resource.
	 * @return Every line in the ini file as an entry in the Map
	 */
	public Map<String, String> loadIniFile(String path) throws IOException;

	/**
	 * Store a string as the contents of a resource.
	 * 
	 * @param path
	 *            relative path to the resource.
	 * @param data
	 *            data to store as the contents of the resource.
	 */
	public void storeString(String path, String data) throws IOException;

	/**
	 * Overwrite the contents of an ini file.
	 * 
	 * @param path
	 *            relative path to the resource.
	 * @param properties
	 *            Every entry in this map will be stored as an entry in the ini file. Keys should not contain '='.
	 */
	public void storeIniFile(String path, Map<String, String> properties) throws IOException;

	/**
	 * Convert a relative path to an absolute path.
	 * 
	 * @param relative
	 *            relative path to the resource.
	 * @return the absolute path on the file system.
	 */
	public String getAbsolutePath(final String relative) throws IOException;

}
