package org.q.app.engine.context;

import java.util.Map;

import com.google.inject.ImplementedBy;

/**
 * Generic interface for storing and retrieving variables. Variables van be stored in memory or persisted to disk.
 * 
 * @author Erik Lievaart
 */
@ImplementedBy(UserVariables.class)
public interface VariableContext {

	/**
	 * Retrieve the value for the given variable key
	 * 
	 * @param name
	 *            name of the variable
	 * @return the value or null if not found.
	 */
	public String getVariable(String name);

	/**
	 * Get a listing of all variables in this context.
	 * 
	 * @return an immutable view on the variables.
	 */
	public Map<String, String> getVariables();

	/**
	 * Set a new value for a variable in memory. The value will be lost when the application is closed.
	 * 
	 * @param name
	 *            Name of the variable. Can be empty "". If the variable does not exist it is created.
	 * @param value
	 *            Value for the variable.
	 */
	public void setVariable(String name, String value);

	/**
	 * Persist a new value for a variable. The value will be available on subsequent runs of the application.
	 * 
	 * @param name
	 *            Name of the variable. Can be empty "". If the variable does not exist it is created.
	 * @param value
	 *            Value for the variable.
	 */
	public void setPermanentVariable(String name, String value);

	/**
	 * Retrieve a variable or return a default value if it is not found.
	 * 
	 * @param name
	 *            Name of the variable.
	 * @param ifAbsent
	 *            value to return if the variable was not found.
	 * @return the value for the variable or the default.
	 */
	public String getVariable(final String name, final String ifAbsent);
}
