package org.q.app.engine.script;

import java.util.Set;

import org.q.app.config.ConfigurableResources;
import org.q.app.config.Resources;
import org.q.app.engine.parse.ShellString;
import org.q.app.ui.bind.ComponentMap;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.Contract;
import org.q.toolkit.lang.Str;

import com.google.inject.Inject;
import com.google.inject.Injector;
import com.google.inject.Provider;

/**
 * This object is injected into scripts to give the scripts access to all aspects of the Q application.
 * 
 * @author Erik Lievaart
 */
@Contract
public class QObject {

	@Inject
	private ComponentMap components;
	@Inject
	private QVariables vars;
	@Inject
	private Provider<QEngineProxy> engine;
	@Inject
	private ConfigurableResources resources;
	@Inject
	private Injector injector;

	private QObject() {
	}

	/**
	 * Get the engine for invoking Q commands.
	 * 
	 * @return a proxy to the engine which can invoke raw code or templated code.
	 */
	public QEngineProxy getEngine() {
		return engine.get();
	}

	/**
	 * Get the {@link QVariables} object that offers access to configuration and location variables.
	 * 
	 * @return the QVariables object.
	 */
	public QVariables getVariables() {
		return vars;
	}

	/**
	 * Get access to system {@link Resources} for advanced use.
	 * 
	 * @return the Resources that offers access to all resources in the application and user configuration directories.
	 */
	public Resources getResources() {
		return resources;
	}

	/**
	 * Add quotes around a String.
	 * 
	 * @param argument
	 *            String to quote.
	 * @return `[argument]`
	 */
	public String quote(final String argument) {
		return Str.quote(argument);
	}

	/**
	 * Advanced functionality for retrieving any class from the Guice injector.
	 * 
	 * @param <E>
	 *            the return type matches the type of the argument.
	 * @param type
	 *            class we want to retrieve the instance for.
	 * @return the object retrieved from Guice.
	 */
	public <E> E getInstance(final Class<E> type) {
		return injector.getInstance(type);
	}

	/**
	 * Create a quoted and escaped shell string literal.
	 * 
	 * @param argument
	 *            Contents of the literal.
	 * @return a String that can be passed as a shell literal to the {@link QEngineProxy}
	 */
	public String quoteAndEscape(final String argument) {
		return quote(ShellString.escape(argument));
	}

	/**
	 * Replace URL escape characters with their escape sequences.
	 * 
	 * @param argument
	 *            String containing escape characters.
	 * @return the URL with the characters escaped.
	 */
	public String urlEscape(final String argument) {
		return UrlTool.escape(argument);
	}

	/**
	 * Get a listing of the id's of all Swing components in use.
	 * 
	 * @return the id's.
	 */
	public Set<String> getComponentIds() {
		return components.getMap().keySet();
	}

	/**
	 * Retrieve a UI component by name.
	 * 
	 * @param name
	 *            Name of the component.
	 * @return the component.
	 */
	public Object getComponent(final String name) {
		return components.getMap().get(name);
	}

}
