package org.q.app.engine.script;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.Map;

import javax.script.ScriptContext;
import javax.script.ScriptEngine;
import javax.script.ScriptEngineManager;
import javax.script.ScriptException;

import org.apache.commons.lang3.Validate;
import org.q.app.contract.PluginContext;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.FormattedException;
import org.q.toolkit.lang.LogTemplate;

import com.google.inject.Inject;
import com.google.inject.Provider;

/**
 * Fine grained access to the script engine for advanced use. This class allows more control over script execution than
 * the {@link SwingScript} class but is more intensive to use. It is strongly recommended to use {@link SwingScript} if
 * possible. Note: unlike the {@link SwingScript} class, this class doesn't necessarily run scripts in the Swing event
 * Thread. This class works by configuring and running scripts in a step by step manner. Always make sure to call one of
 * the createEngine methods before invoking any script code.
 * 
 * @author Erik Lievaart
 */
public class ScriptInvoker {
	private final LogTemplate log = new LogTemplate(getClass());

	@Inject
	private ScriptCache resources;
	@Inject
	private Provider<QObject> q;
	@Inject
	private Provider<PluginContext> contextProvider;

	private ScriptEngine engine;
	private String extension;

	/**
	 * Create the engine with a default {@link ScriptConfig}.
	 */
	public void createEngine() {
		createEngine(new ScriptConfig());
	}

	/**
	 * Create the engine with the supplied {@link ScriptConfig}.
	 * 
	 * @param config
	 *            customize the script environment.
	 */
	public void createEngine(final ScriptConfig config) {
		extension = config.getExtension();
		engine = new ScriptEngineManager().getEngineByExtension(extension);
		Validate.notNull("Script extension % not registered!", config.getExtension());

		QObject qObject = q.get();
		if (config.isDefaultProperties()) {
			qObject.getVariables().putAll(contextProvider.get().getVariables());
		}
		engine.getContext().setAttribute("q", qObject, ScriptContext.ENGINE_SCOPE);

		Map<String, Object> properties = config.getUserProperties();
		for (String name : properties.keySet()) {
			engine.getContext().setAttribute(name, properties.get(name), ScriptContext.ENGINE_SCOPE);
		}
	}

	/**
	 * Run a script resource.
	 * 
	 * @param <E>
	 *            automatically cast the return value to the assigned type.
	 * @param resource
	 *            base name of the script.
	 * @return the return value if any.
	 */
	@SuppressWarnings("unchecked")
	public <E> E runResource(final String resource) {
		checkEngine();
		Check.isFalse(resource.contains("/"), "`/` not allowed in %", resource);
		try {
			return (E) engine.eval(resources.readResource(resource));
		} catch (Exception e) {
			throw new FormattedException("Error % Unable to run script file %", e, e.getMessage(), resource);
		}
	}

	/**
	 * Run a {@link File} as script.
	 * 
	 * @param <E>
	 *            automatically cast the return value to the assigned type.
	 * @param file
	 *            a File on the file system containing code.
	 * @return the return value if any.
	 * @throws FileNotFoundException
	 *             If the file is not on the File system.
	 * @throws ScriptException
	 *             Low level Exception signaling problems in the script.
	 */
	@SuppressWarnings("unchecked")
	public <E> E runFile(final File file) throws FileNotFoundException, ScriptException {
		checkEngine();
		Check.isTrue(file.exists(), "file does not exist: %", file);
		log.trace("Loading script file %", file.getPath());
		return (E) engine.eval(new BufferedReader(new FileReader(file)));

	}

	/**
	 * Exaluate raw script code.
	 * 
	 * @param <E>
	 *            automatically cast the return value to the assigned type.
	 * @param code
	 *            raw script code to execute.
	 * @return the return value if any.
	 */
	@SuppressWarnings("unchecked")
	public <E> E eval(final String code) {
		checkEngine();
		try {
			return (E) engine.eval(code);
		} catch (ScriptException e) {
			throw new RuntimeException(e);
		}
	}

	private void checkEngine() {
		Check.notNull(engine, "Call createEngine first!");
	}

}
