package org.q.app.ui.context;

import java.io.IOException;
import java.util.Map;

import org.apache.commons.vfs.FileObject;
import org.q.app.config.ResourcePaths;
import org.q.app.engine.script.ScriptConfig;
import org.q.app.engine.script.ScriptInvocation;
import org.q.app.engine.script.SwingScript;
import org.q.app.vfs.UrlResolver;
import org.q.app.vfs.VirtualFile;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.lang.collection.NewCollection;

import com.google.inject.Inject;

/**
 * Dispatches navigation events in the file browser lists.
 * 
 * @author Erik Lievaart
 */
public class BrowseEvents {

	@Inject
	private ContextMediator switcher;
	@Inject
	private SwingScript scripts;

	/**
	 * Refresh both file browser lists.
	 */
	public void refresh() {
		Map<String, Object> properties = NewCollection.map();
		properties.put("left", switcher.getLeft().getDir());
		properties.put("right", switcher.getLeft().getDir());

		ScriptInvocation invocation = new ScriptInvocation();
		invocation.addResource(ResourcePaths.UI_REQUIRES_REFRESH_SCRIPT);
		invocation.addResource(ResourcePaths.UI_SHOW_REFRESH_SCRIPT);
		scripts.invoke(new ScriptConfig(properties), invocation);
	}

	/**
	 * Swap the contents of the left and right file browser lists.
	 */
	public void swap() {
		VirtualFile oldLeft = switcher.getLeft().getDir();
		VirtualFile oldRight = switcher.getRight().getDir();

		setLocation(Orientation.LEFT, oldRight);
		setLocation(Orientation.RIGHT, oldLeft);
	}

	/**
	 * Open an URL in a file browser list.
	 * 
	 * @param orientation
	 *            Specifies the file browser list that should open the URL.
	 * @param location
	 *            absolute or relative URL to open.
	 * @param base
	 *            base URL to use if the location is relative.
	 */
	public void url(final String orientation, final String location, final FileObject base) {
		try {
			setLocation(getOrientation(orientation), UrlResolver.resolve(location, base));
		} catch (IOException e) {
			throw new RuntimeIOException("Unable to open location: " + location, e);
		}
	}

	/**
	 * Open a fuzzy URL in a file browser list.
	 * 
	 * @param orientation
	 *            Specifies the file browser list that should open the URL.
	 * @param location
	 *            fuzzy URL to open.
	 * @param base
	 *            base URL to use if the location is relative.
	 */
	public void fuzzy(final String orientation, final String location, final FileObject base) throws IOException {
		setLocation(getOrientation(orientation), UrlResolver.resolveFuzzy(location, base));
	}

	private Orientation getOrientation(final String orientation) {
		return Orientation.valueOf(orientation.toUpperCase());
	}

	private void setLocation(final Orientation orientation, final VirtualFile right) {
		Map<String, Object> properties = NewCollection.map();
		properties.put("orientation", orientation.getBrowserName(switcher));
		properties.put("location", right);

		scripts.invoke(new ScriptConfig(properties), new ScriptInvocation(ResourcePaths.UI_LOCATION_SCRIPT));
	}

	@Override
	public String toString() {
		return getClass().getSimpleName();
	}

	private enum Orientation {
		ACTIVE {
			@Override
			public String getBrowserName(final ContextMediator switcher) {
				Orientation orientation = switcher.getActive() == switcher.getLeft() ? LEFT : RIGHT;
				return orientation.getBrowserName(switcher);
			}
		},
		INACTIVE {
			@Override
			public String getBrowserName(final ContextMediator switcher) {
				Orientation orientation = switcher.getActive() == switcher.getLeft() ? RIGHT : LEFT;
				return orientation.getBrowserName(switcher);
			}
		},
		LEFT {
			@Override
			public String getBrowserName(final ContextMediator switcher) {
				return "left";
			}
		},
		RIGHT {
			@Override
			public String getBrowserName(final ContextMediator switcher) {
				return "right";
			}
		};

		public abstract String getBrowserName(ContextMediator switcher);
	}

}
