package org.q.app.ui.context;

import java.util.Map;

import org.q.app.engine.context.Active;
import org.q.app.engine.context.EngineSnapshot;

import com.google.inject.Inject;
import com.google.inject.Singleton;

/**
 * Context object that registers the locations opened in the left and right file browser lists. Also keeps track of
 * which browser list is active (last selected).
 * 
 * @author Erik Lievaart
 */
@Singleton
public class ContextMediator {

	@Inject
	private EngineSnapshotScript engineSnapshotScript;

	private Active active = Active.LEFT;
	private final UIContext left = new UIContext();
	private final UIContext right = new UIContext();

	private ContextMediator() {
	}

	/**
	 * Get the {@link UIContext} for the active file browser list.
	 */
	public UIContext getActive() {
		return active == Active.LEFT ? left : right;
	}

	/**
	 * Get the {@link UIContext} for the inactive file browser list.
	 */
	public UIContext getInactive() {
		return active == Active.LEFT ? right : left;
	}

	/**
	 * Get the {@link UIContext} for the left file browser list.
	 */
	public UIContext getLeft() {
		return left;
	}

	/**
	 * Get the {@link UIContext} for the right file browser list.
	 */
	public UIContext getRight() {
		return right;
	}

	/**
	 * Mark the left file browser list as active.
	 */
	public void setLeftActive() {
		active = Active.LEFT;
	}

	/**
	 * Mark the right file browser list as active.
	 */
	public void setRightActive() {
		active = Active.RIGHT;
	}
	
	/**
	 * Is the right browser window active.
	 */
	public boolean isLeftActive(){
		return active == Active.LEFT;
	}


	private EngineSnapshot getEngineSnapshot() {
		return engineSnapshotScript.invoke(active, left.getDir(), right.getDir());
	}

	/**
	 * Get a variable map containing all the engine variables. Examples: $dir, $urls, etc.
	 * 
	 * @return they keys are the names of the variables (without a '$' sign), the values are the escaped URL's.
	 */
	public Map<String, String> getEngineVariables() {
		return EngineVariables.resolve(getEngineSnapshot());
	}

}
