package org.q.app.ui.render.browser;

import org.q.app.config.VariableConstants;
import org.q.app.engine.context.UserVariables;
import org.q.app.ui.render.list.IconFactory;
import org.q.app.ui.render.list.NoColorFactory;
import org.q.app.ui.render.list.NoIconFactory;
import org.q.app.ui.render.list.QListCellRenderer;

import com.google.inject.Inject;
import com.google.inject.Provider;
import com.google.inject.Singleton;

/**
 * This class provides a configurable {@link QListCellRenderer}. It can turn on and off the colors, and icons for
 * {@link javax.swing.JList}'s that use this {@link QListCellRenderer}.
 * 
 * @author Erik Lievaart
 */
@Singleton
public class VirtualFileCellController {

	private final QListCellRenderer renderer = new QListCellRenderer();
	private final UserVariables settings;

	// Using a static field is pretty ugly, but otherwise I can't use it from the enum...
	private static Provider<LocalIconFactory> localProvider;

	@Inject
	VirtualFileCellController(final UserVariables settings, final Provider<LocalIconFactory> localProvider) {
		this.settings = settings;
		VirtualFileCellController.localProvider = localProvider;

		showIcons(Icons.valueOf(settings.getVariable(VariableConstants.BROWSER_ICONS, "none").toUpperCase()));

		setColorFactory(Boolean.parseBoolean(settings.getVariable(VariableConstants.BROWSER_COLOR, "true")));
	}

	/**
	 * Get the configurable renderer.
	 */
	public QListCellRenderer getRenderer() {
		return renderer;
	}

	/**
	 * Display files in the {@link javax.swing.JList} in color.
	 */
	public void displayColor() {
		setColorFactory(true);
		settings.setPermanentVariable(VariableConstants.BROWSER_COLOR, "true");
	}

	/**
	 * Display files in the {@link javax.swing.JList} in monochrome.
	 */
	public void displayMonochrome() {
		setColorFactory(false);
		settings.setPermanentVariable(VariableConstants.BROWSER_COLOR, "false");
	}

	private void setColorFactory(final boolean useColor) {
		renderer.setForegroundFactory(useColor ? new VirtualFileColorFactory() : new NoColorFactory());
	}

	/**
	 * Display files in the {@link javax.swing.JList} using local icons.
	 */
	public void displayLocalIcons() {
		showIcons(Icons.LOCAL);
	}

	/**
	 * Display files in the {@link javax.swing.JList} without icons.
	 */
	public void displayNoIcons() {
		showIcons(Icons.NONE);
	}

	/**
	 * Display files in the {@link javax.swing.JList} with icons using the
	 * {@link javax.swing.filechooser.FileSystemView} class.
	 */
	public void displayOsIcons() {
		showIcons(Icons.OS);
	}

	private void showIcons(final Icons icons) {
		renderer.setIconFactory(icons.getFactory());
		settings.setPermanentVariable(VariableConstants.BROWSER_ICONS, icons.name());
	}

	private enum Icons {
		NONE {
			@Override
			public IconFactory getFactory() {
				return new NoIconFactory();
			}
		},
		LOCAL {
			@Override
			public IconFactory getFactory() {
				return localProvider.get();
			}
		},
		OS {
			@Override
			public IconFactory getFactory() {
				return new OsIconFactory();
			}
		};

		public abstract IconFactory getFactory();
	}
}
