package org.q.app.vfs;

import java.io.IOException;

import org.apache.commons.vfs.FileObject;
import org.apache.commons.vfs.VFS;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.concurrent.Stateless;

/**
 * Resolve {@link VirtualFile}'s from URL's.
 * 
 * @author Erik Lievaart
 */
@Stateless
public class UrlResolver {

	private UrlResolver() {
	}

	/**
	 * Resolve a {@link VirtualFile} using a relative or absolute URL relative to a base URL.
	 * 
	 * @param url
	 *            location to resolve.
	 * @param base
	 *            Resolve location relative to this base URL.
	 * @return the resolved VirtualFile.
	 * @throws IOException
	 *             if unable to resolve.
	 * @see VFS
	 */
	public static VirtualFile resolve(final String url, final FileObject base) throws IOException {
		Check.notNull(url, "Cannot resolve <null>");
		return new CommonsFile(VFS.getManager().resolveFile(base, url));
	}

	/**
	 * Resolve an absolute URL.
	 * 
	 * @param url
	 *            exact URL to a location.
	 * @return the resolved VirtualFile.
	 * @throws IOException
	 *             if unable to resolve.
	 * @see VFS
	 */
	public static VirtualFile resolve(final String url) throws IOException {
		return resolve(url, (FileObject) null);
	}

	/**
	 * Use a fuzzy matching algorithm to resolve a URL. For each part of the URL between slashes an exact match will be
	 * attempted and otherwise a fuzzy match will be attempted. Wildcards '*' and '?' can be used. If none is specified,
	 * a trailing '*' will be inserted.
	 * 
	 * @param url
	 *            location to resolve.
	 * @param base
	 *            Resolve location relative to this base URL.
	 * @return the resolved URL.
	 * @throws IOException
	 *             if unable to resolve.
	 * @see VFS
	 */
	public static VirtualFile resolveFuzzy(final String url, final FileObject base) throws IOException {
		return FuzzyUrlResolver.resolveFuzzy(url, base);
	}

}
