package org.q.app.vfs;

import java.io.IOException;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.io.IOUtils;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.io.UrlTool;
import org.q.toolkit.lang.Check;
import org.q.toolkit.lang.Contract;
import org.q.toolkit.lang.Str;
import org.q.toolkit.lang.collection.NewCollection;

/**
 * Utility class for working with {@link VirtualFile}'s.
 * 
 * @author Erik Lievaart
 */
@Contract
public class VirtualFiles {

	private static final VirtualFile[] EMPTY = new VirtualFile[] {};

	private VirtualFiles() {
	}

	static VirtualFile findNamedChild(final VirtualFile[] files, final String name) {
		for (VirtualFile file : files) {
			if (file.getBaseName().equals(name)) {
				return file;
			}
		}
		throw new IllegalStateException(Str.sub("Child with name % not found in %", name, files));
	}

	static TreeSet<String> getNamesAlphabetically(final VirtualFile[] files) {
		TreeSet<String> names = new TreeSet<String>(new VirtualFileNameComparator());
		for (VirtualFile file : files) {
			names.add(file.getBaseName());
		}
		return names;
	}

	/**
	 * Return the children of a {@link VirtualFile} in order.
	 * 
	 * @param file
	 *            parent which' children to retrieve.
	 * @return the children in order.
	 * @see VirtualFileComparator
	 */
	public static VirtualFile[] getChildrenAlphabetically(final VirtualFile file) {
		VirtualFile[] children = file.listFiles();
		Arrays.sort(children, new VirtualFileComparator());
		return children;
	}

	static VirtualFile[] getDirsAlphabetically(final VirtualFile file) {
		TreeSet<VirtualFile> sorted = new TreeSet<VirtualFile>(new VirtualFileComparator());

		for (VirtualFile child : file.listFiles()) {
			if (child.isDirectory()) {
				sorted.add(child);
			}
		}
		return sorted.toArray(EMPTY);
	}

	/**
	 * Return the {@link VirtualFile} if it exists, otherwise return the longest
	 * existing parent path.
	 * 
	 * @param child
	 *            VirtualFile we hope exists.
	 */
	public static VirtualFile getLongestExistingPath(final VirtualFile child) {
		if (child == null || child.exists()) {
			return child;
		}
		return getLongestExistingPath(child.getParentFile());
	}

	/**
	 * Find duplicate names in a {@link Collection} of {@link VirtualFile}'s.
	 * This can be useful, for instance, when copying files. If there are
	 * duplicate file name, we cannot copy two files in the collection to the
	 * same directory without overwriting one of the two.
	 * 
	 * @param files
	 *            Collection of files that may contain a duplicate name.
	 * @return the first duplicate name found or null if none was found.
	 */
	public static String findDuplicateFileName(final Collection<? extends VirtualFile> files) {
		Set<String> names = NewCollection.set();

		for (VirtualFile file : files) {
			if (!names.add(file.getNameEscaped())) {
				return file.getName();
			}
		}
		return null;
	}

	/**
	 * Return a List of all files nested at any depth.
	 * 
	 * @param root
	 *            root directory to start looking from.
	 * @return all nested files (no directories).
	 */
	public static List<VirtualFile> getAllContainedFiles(final VirtualFile root) {
		Check.isTrue(root.isDirectory(), "Cannot list contained files of %, not a directory", root);
		List<VirtualFile> all = NewCollection.list();

		for (VirtualFile file : root.listFiles()) {
			if (file.isFile()) {
				all.add(file);
			}
			if (file.isDirectory()) {
				all.addAll(getAllContainedFiles(file));
			}
		}
		return all;
	}

	/**
	 * Read a VirtualFile to String.
	 * 
	 * @param file
	 *            File to read.
	 * @return the contents.
	 */
	public static String toString(final VirtualFile file) {
		try {
			return IOUtils.toString(file.getFileObject().getContent().getInputStream());
		} catch (IOException e) {
			throw new RuntimeIOException(e);
		}
	}

	/**
	 * Read a VirtualFile to String.
	 * 
	 * @param file
	 *            File to read.
	 * @param encoding
	 *            file encoding.
	 * @return the contents.
	 */
	public static String toString(final VirtualFile file, final String encoding) {
		try {
			return IOUtils.toString(file.getFileObject().getContent().getInputStream(), encoding);
		} catch (IOException e) {
			throw new RuntimeIOException(e);
		}
	}

	/**
	 * Returns true iff both files refer to the same url.
	 */
	public static boolean isSameUrl(final VirtualFile first, final VirtualFile second) {
		Check.notNull(first, second);
		return first.getUrl().equals(second.getUrl());
	}

	/**
	 * Returns true iff isParent is a parent of child.
	 */
	public static boolean isParentOf(final VirtualFile isParent, final VirtualFile child) {
		Check.notNull(isParent, child);

		if (isParent.getUrl().equals(child.getUrl())) {
			return true;
		}
		String parentOfChild = UrlTool.getParent(child.getUrl());
		return UrlTool.addSlash(parentOfChild).startsWith(UrlTool.addSlash(isParent.getUrl()));
	}

}
