package org.q.plugins.delete;

import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.vfs.AllFileSelector;
import org.apache.commons.vfs.FileSystemException;
import org.q.app.contract.Invocable;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.ui.context.BrowseEvents;
import org.q.app.vfs.VirtualFile;
import org.q.toolkit.lang.AssertionException;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.QUEUE)
@Doc("Delete files on any supported Filesystem.")
class DeleteShellCommand implements Invocable {

	private final Log log = LogFactory.getLog(getClass());
	private VirtualFile file;
	private List<VirtualFile> urls;
	private Mode mode;

	@Inject
	BrowseEvents events;
	@Inject
	TrashCache cache;

	private enum Mode {
		SINGLE, URLS
	}

	@Flag(group = "main", values = { "$url" })
	@Doc("Delete a single file or directory. One argument: url to delete")
	public DeleteShellCommand single(final VirtualFile file) {
		this.file = file;
		mode = Mode.SINGLE;
		return this;
	}

	@Flag(group = "main", values = { "$urls" }, primary = true)
	@Doc("Delete multiple urls. One argument: urls to delete")
	public DeleteShellCommand urls(final List<VirtualFile> urls) {
		this.urls = urls;

		mode = Mode.URLS;
		return this;
	}

	@Override
	public void validate() throws ValidationException {
	}

	@Override
	public void invoke() throws FileSystemException {
		switch (mode) {
			case SINGLE:
				deleteSingle(file);
				return;

			case URLS:
				deleteUrls();
				return;

			default:
				throw new AssertionException("Unknown enum constant: %", mode);
		}
	}

	private void deleteUrls() throws FileSystemException {
		for (VirtualFile vf : urls) {
			deleteSingle(vf);
		}
	}

	private void deleteSingle(final VirtualFile remove) throws FileSystemException {
		VirtualFile trash = cache.getTrashLocation(remove);
		if (trash != null && !remove.getUrl().contains(TrashCache.TRASH_NAME)) {
			if (!trash.exists()) {
				trash.getFileObject().createFolder(); // files dir may have been deleted
			}
			VirtualFile deleteLocation = trash.resolveFile(remove.getName());
			remove.getFileObject().moveTo(deleteLocation.getFileObject());
		}
		int count = remove.getFileObject().delete(new AllFileSelector());

		log.trace(remove.getUrl() + " deleted: " + count);
		events.refresh();
	}
}
