package org.q.plugins.find;

import java.util.Hashtable;
import java.util.Map;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.q.app.config.ResourcePaths;
import org.q.app.contract.Invocable;
import org.q.app.contract.Swallows;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.engine.script.ScriptConfig;
import org.q.app.engine.script.ScriptInvocation;
import org.q.app.engine.script.SwingScript;
import org.q.app.vfs.VirtualFile;
import org.q.toolkit.lang.Check;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.SINGLE)
@Doc("Find files or directories using patterns or regex")
class FindShellCommand implements Invocable, Swallows {

	private final FileFinder finder = new FileFinder();

	@Inject
	private SwingScript script;
	@Inject
	private IncrementalFind incremental;

	@Flag(group = "location", primary = true, values = "$dir")
	@Doc("Location to find files in. 1 argument: the directory")
	public FindShellCommand location(final VirtualFile dir) {
		finder.setDir(dir);
		return this;
	}

	@Flag(group = "excludes", values = "``")
	@Doc("Wildcard expression to exclude file names from the match results")
	public FindShellCommand excludes(final String pattern) {
		finder.setExcludes(pattern);
		return this;
	}

	@Flag(group = "types")
	@Doc("Look for directories only")
	public FindShellCommand directories() {
		finder.onlyDirectories();
		return this;
	}

	@Flag(group = "types")
	@Doc("Look for files only")
	public FindShellCommand files() {
		finder.onlyFiles();
		return this;
	}

	@Flag(group = "includes", values = "`*`", primary = true)
	@Doc("Wildcard expression to match file name")
	public FindShellCommand includes(final String pattern) {
		finder.setIncludes(pattern);
		return this;
	}

	@Flag(group = "local")
	@Doc("Search current dir only")
	public FindShellCommand noRecurse() {
		finder.searchLocal();
		return this;
	}

	@Flag(group = "paths")
	@Doc("Use regex against paths instead of filenames only.")
	public FindShellCommand pathRegex() {
		finder.setPathRegex();
		return this;
	}

	@Override
	public void invoke() throws Exception {
		init();

		Map<String, Object> properties = new Hashtable<String, Object>();
		properties.put("dir", finder.getDir());
		properties.put("matches", finder.match());
		properties.put("MAX_RESULTS", FileFinder.MAX_RESULTS);

		ScriptInvocation si = new ScriptInvocation(ResourcePaths.UI_POPUP_SCRIPT, ResourcePaths.FIND_RESULT_SCRIPT);
		script.invoke(new ScriptConfig(properties), si);
	}

	private void init() {
		script.invoke(new ScriptInvocation(ResourcePaths.FIND_START_SCRIPT));

		incremental.setDir(finder.getDir());
		finder.addObserver(incremental);
	}

	@Override
	public void validate() throws ValidationException {
		Check.isFalse(finder.isPathRegex(), "Expecting a swallowed regex when -p flag is set!");
	}

	@Override
	@Doc("Return only files matching the swallowed regular expression (not required).")
	public void invoke(final String raw) throws Exception {
		finder.setRegex(Pattern.compile(raw));
		invoke();
	}

	@Override
	public void validate(final String raw) throws ValidationException {
		try {
			Pattern.compile(raw);
		} catch (PatternSyntaxException pse) {
			throw new ValidationException(String.format("Pattern '%s' does not compile: %s", raw, pse.getMessage()));
		}
	}

}
