package org.q.plugins.move;

import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.commons.vfs.AllFileSelector;
import org.apache.commons.vfs.FileObject;
import org.apache.commons.vfs.FileSystemException;
import org.q.app.contract.Invocable;
import org.q.app.contract.ThreadPolicy;
import org.q.app.contract.ValidationException;
import org.q.app.contract.annotation.Concurrency;
import org.q.app.contract.annotation.Doc;
import org.q.app.contract.annotation.Flag;
import org.q.app.ui.context.BrowseEvents;
import org.q.app.vfs.VirtualFile;
import org.q.app.vfs.VirtualFiles;
import org.q.toolkit.io.RuntimeIOException;
import org.q.toolkit.lang.FormattedException;

import com.google.inject.Inject;

@Concurrency(ThreadPolicy.QUEUE)
@Doc("move one or more files from any filesystem to any writable filesystem")
class MoveShellCommand implements Invocable {
	private static final String MOVE_TO_CHILD_MSG = "% => %; cannot move to a child of itself";

	@Inject
	BrowseEvents events;

	private final Log log = LogFactory.getLog(getClass());
	private VirtualFile source;
	private VirtualFile destination;
	private String rename;
	private Mode mode;

	private List<VirtualFile> urls;

	@Flag(group = "main", values = { "$url", "$dir~", "``" })
	@Doc("Move a file or directory. Three arguments: 1) file to move 2) new parent. 3) new file name (if not empty)")
	public MoveShellCommand single(final VirtualFile file, final VirtualFile dir, final String name) {
		source = file;
		destination = dir;
		rename = name;

		mode = Mode.SINGLE;
		return this;
	}

	@Flag(group = "main", values = { "$urls", "$dir~" }, primary = true)
	@Doc("Move multiple urls. Two arguments 1) urls to move 2) directory to move urls into")
	public MoveShellCommand urls(final List<VirtualFile> urls, final VirtualFile destination) {
		this.destination = destination;
		this.urls = urls;

		mode = Mode.URLS;
		return this;
	}

	@Override
	public void invoke() throws FileSystemException {
		switch (mode) {

		case SINGLE:
			moveSingleFile();
			return;

		case URLS:
			destination.getFileObject().createFolder();
			moveUrls();
			return;

		default:
			throw new RuntimeException("Unknown enum constant: " + mode);
		}
	}

	private void moveUrls() throws FileSystemException {
		for (VirtualFile file : urls) {
			moveVirtualFile(file, destination.resolveFile(file.getNameEscaped()));
			events.refresh();
		}
	}

	private void moveSingleFile() throws FileSystemException {
		String name = StringUtils.defaultIfEmpty(rename, source.getNameEscaped());
		moveVirtualFile(source, destination.resolveFile(name));
		events.refresh();
	}

	private void moveVirtualFile(final VirtualFile source, final VirtualFile destination) throws FileSystemException {
		if (VirtualFiles.isSameUrl(source, destination)) {
			return;
		}
		if (VirtualFiles.isParentOf(source, destination)) {
			throw new RuntimeIOException(MOVE_TO_CHILD_MSG, source.getUrl(), destination.getUrl());
		}
		if (log.isTraceEnabled()) {
			log.trace(source + " => " + destination);
		}
		if (!source.isDirectory() || !destination.exists()) {
			moveFile(source.getFileObject(), destination.getFileObject());
			return;
		}
		for (VirtualFile child : source.listFiles()) {
			moveVirtualFile(child, destination.resolveFile(child.getNameEscaped()));
		}
		source.getFileObject().delete();
	}

	private void moveFile(final FileObject source, final FileObject destination) throws FileSystemException {
		try {
			if (!destination.exists()) {
				source.moveTo(destination);
				return;
			}
		} catch (FileSystemException fse) {
		}
		destination.copyFrom(source, new AllFileSelector());
		source.delete(new AllFileSelector());
	}

	@Override
	public void validate() throws ValidationException {
		try {
			ValidationException.unless(destination.getFileObject().isWriteable(), "Not writable: %", destination);

			switch (mode) {
			case URLS:
				validateUrls();
				return;

			case SINGLE:
				ValidationException.unless(source.exists(), "File does not exist: %", source);
				return;

			default:
				throw new FormattedException("% unknown enum constant", mode);
			}

		} catch (FileSystemException e) {
			throw new ValidationException("Unable to read filesystem; " + e.getMessage(), e);
		}
	}

	private void validateUrls() throws ValidationException {
		String duplicate = VirtualFiles.findDuplicateFileName(urls);
		ValidationException.unless(duplicate == null, "duplicate name: %", duplicate);
	}

	private enum Mode {
		SINGLE, URLS;
	}

}
